% Controller design tool
echo off;

% Ejs variable: Static gain of the plant: ball and plate
k_ejs = 9.81*(5.0/7.0);
% Matlab variable: Static gain of the plant
k = k_ejs;


delay = 0;
kpmax = 8;
kimax = 8;
nw = 200;
wmin = 0.01;
wmax = 100;
wdesignmin = wmin;
wdesignmax = wmax;

magmin = -40;
magmax = 40;
phasemin = -190;
phasemax = 180;

c = 1;
escalaNyq = 1;
integ = 0;
epsilon = 0.00001;

ww = logspace(log10(wmin), log10(wmax), nw);

% Plant: Pure integrator
nump = k;
denp = [1 0];

% Plant: Double integrator
nump_ejs=k;
denp_ejs=[1 0 0];

[ro,argG] = argumentoG(k,nump,denp,integ,delay,wdesign);
[kp,ti,argderL,teta] = TuningModulus(M,k,nump,denp,integ,argG,ro,wdesign,epsilon,c);

% Equivalence between PI*(1/s) and PD*(1(s^2) parameters
td_ejs =  ti;
kp_ejs = kp / td_ejs;

% PI controller 
numrPI = [kp*ti kp];
denrPI = [ti 0];
 
% PD cotroller
numrPD = [kp_ejs*td_ejs kp_ejs];
denrPD = [0 1];

% Loop transfer function of the PI*(1/s)
numlaPI = [k*kp*ti k*kp];
denlaPI = [ti 0 0];

% Loop transfer function of the PD*(1/s^2): Similar to the previous one
%numlaPD = [k*kp_ejs*td_ejs k*kp_ejs];
%denlaPD = [1 0 0];

tf_p = tf(nump, denp);
tf_p_ejs = tf(nump_ejs, denp_ejs);
tf_rPI = tf(numrPI, denrPI);
tf_rPD = tf(numrPD, denrPD);
tf_laPI = tf(numlaPI, denlaPI);

[Gm,Pm,Wcg,Wcp] = margin (tf_laPI);
gm_ejs = Gm;
pm_ejs = Pm;
wcg_ejs = Wcg;
wcp_ejs = Wcp;

[mag, phase] = bode (tf_p, ww);
[mag_ejs, phase_ejs] = bode (tf_p_ejs, ww);
[mag_rPI, phase_rPI] = bode (tf_rPI, ww);
[mag_rPD, phase_rPD] = bode (tf_rPD, ww);
[mag_laPI, phase_laPI] = bode (tf_laPI, ww);

magdb = 20*log10(mag);
magdb_ejs = 20*log10(mag_ejs);
magdb_rPI = 20*log10(mag_rPI);
magdb_rPD = 20*log10(mag_rPD);
magdb_laPI = 20*log10(mag_laPI);

[re, im] = nyquist(tf_p, ww);
[re_ejs, im_ejs] = nyquist(tf_p_ejs, ww);
[re_rPI, im_rPI] = nyquist(tf_rPI, ww);
[re_rPD, im_rPD] = nyquist(tf_rPD, ww);
[re_laPI, im_laPI] = nyquist(tf_laPI, ww); 


subplot(2, 2, 1,'position',[0.08 0.58 0.41 0.4],'box','on'),...
%% Begin Bode amplitude   
   xw = [wdesignmin wdesignmin wdesignmax wdesignmax wdesignmin];
   yw = [100*magmin 100*magmax 100*magmax magmin magmin];
   semilogx (ww, magdb (1,:), 'r'),...  % Necesario para fijar los ejes 
   axis([ww(1),ww(end),magmin,magmax]),...
   hold on,...
   fill(xw,yw, 'y'),...
   %semilogx (ww, magdb (1,:),'r',ww, magdb_rPI (1,:),'b',ww, magdb_laPI (1,:),'k','LineWidth',2),...
   semilogx (ww, magdb_ejs (1,:),'r',ww, magdb_rPD (1,:),'b',ww, magdb_laPI (1,:),'k','LineWidth',2),...
   line([wdesign wdesign], [magmin magmax], 'Color','m','LineWidth',2, 'LineStyle','-.'),...
   line([wdesignmin wdesignmax], [1 1], 'Color','c','LineWidth',1),...
   xlabel('w (rd/sg)'),...
   ylabel('dB'),... 
   hold off;
%% End Bode amplitude    

subplot(2, 2, 2,'position',[0.58 0.58 0.41 0.4],'box','on');
%% Begin Bode phase    
  xw = [wdesignmin wdesignmin wdesignmax wdesignmax wdesignmin];
  yw = [-30+phasemin phasemax+30 phasemax+30 -30+phasemin -30+phasemin];
%  semilogx (ww, phase (1,:),'r'),... % Necesario para fijar los ejes
  semilogx (ww, phase_ejs (1,:),'r'),... % Necesario para fijar los ejes
  axis([ww(1),ww(end),-30+phasemin,phasemax+30]),...
  hold on,...
  fill (xw, yw, 'y'),...
%  semilogx (ww, phase (1,:),'r',ww, phase_rPI (1,:),'b',ww, phase_laPI (1,:),'k','LineWidth',2),...
  semilogx (ww, phase_ejs (1,:),'r',ww, phase_rPD (1,:),'b',ww, phase_laPI (1,:),'k','LineWidth',2),...
  plot([wdesign wdesign], [-30+phasemin phasemax+30], 'Color','m','LineWidth',2, 'LineStyle','-.'),...
  plot([wdesignmin wdesignmax], [-180 -180], 'Color','c','LineWidth',1),...  
  xlabel('w (rd/sg)'),...
  ylabel('Ph ()'),...
  hold off;
% End Bode phase  


subplot(2, 2, 3,'position',[0.08 0.09 0.41 0.4],'box','on'),...
% Begin Nyquist plot
  axis(escalaNyq*[-2.5,2.5,-2.5,1.5]),... 
  hold on;
  sc = axis;
  
  fiB = atan(M*sin(teta*pi/180)/(c-M*cos(teta*pi/180)))*180/pi;
  xC = [0];
  yC = [0];
  for j = 1:11
       xC = [xC 10*escalaNyq*sin(pi*(0.5+(fiB+90+9*(j-1))/180))];
       yC = [yC -10*escalaNyq*cos(pi*(0.5+(fiB+90+9*(j-1))/180))];
  end
  xC = [xC 0];
  yC = [yC 0];
  
  x = M * cos (0:0.01:2*pi)-c;
  y = M * sin (0:0.01:2*pi);
  fill(xC, yC,'y'),...  
  fill (x, y, 'g','edgeColor',[0 1 0]),...
  line(escalaNyq*[-2.5 2.5], [0 0],'Color','k'),... 
  line([0 0], escalaNyq*[-2.5 1.5],'Color','k'),...
  plot(-c+M*cos(teta*pi/180),-M*sin(teta*pi/180),'Ko','MarkerSize',2),...
  plot (re_ejs(1,:), im_ejs(1,:), 'R', re_laPI(1,:), im_laPI(1,:), 'K', re_rPD(1,:), im_rPD(1,:),'B', 'LineWidth',2);
%%plot (re(1,:), im(1,:), 'R', re_laPI(1,:), im_laPI(1,:), 'K', re_rPI(1,:), im_rPI(1,:),'B', 'LineWidth',2);
%%  plot (re(1,:), im(1,:), 'R', re_laPI(1,:), im_laPI(1,:), 'K','LineWidth',2);
  
  x = cos (0:0.01:2*pi);
  y = sin (0:0.01:2*pi);
  plot (x, y, 'm');

  [magB, faseB]= bode(numlaPI,denlaPI,wdesign);
  faseB = faseB-delay*wdesign;
  plot(-k/wdesign*sin(delay*wdesign),-k/wdesign*cos(delay*wdesign),'Ko','MarkerSize',2,'LineWidth',1);

  %Dibuja la pendiente de L(jwdesign) en wdesign
  [re_wdesign, im_wdesign] = nyquist(tf_laPI,wdesign);
  Lx = re_wdesign * cos(delay*wdesign) + im_wdesign * sin(delay*wdesign);
  Ly = -re_wdesign * sin(delay*wdesign) + im_wdesign * cos(delay*wdesign);
  fpendienteL = @(x) -tan(argderL)*Lx+Ly+tan(argderL)*x;
  line ([-2.5 2.5], [fpendienteL(-2.5) fpendienteL(2.5)], 'Color', 'b'),...
  xlabel('Re'),...
  ylabel('Im'),...
  hold off;
% End Nyquist plot


subplot(2, 2, 4,'color','b','position',[0.58 0.09 0.41 0.4],'box','on'),...
% Begin Robustness region  
  axis([0,kpmax,0,kimax]),... 
  hold on;
  wwR = ww;
  fi = -(pi/2)-wwR*delay;
  ro = k./wwR;
  fiwdesign = -(pi/2)-wdesign*delay;
  rowdesign = k/wdesign;
  kpM = 0:kpmax/60:kpmax;
  kiM = k*c*(1+sqrt(1-(M*M/(c*c))))/(2*M*M)*(kpM.*kpM);
  xC = [0 0 kimax kpmax 0];
  yC = [0 kimax kpmax 0 0];
  fill ([kpM 0], [kiM kiM(end)], 'w', 'edgeColor',[1 1 1]), ...
  plot (kpM, kiM, 'r',kp, kp/ti,'xR', 'MarkerSize', 10, 'LineWidth',2),...
  line([0 kpmax],[0 0],'Color','k'), line([0 0],[0 kimax],'Color','k'),...
  xlabel('Kp'), ylabel ('Ki'), ...
  hold off;
% End Robustness region

set (gcf, 'Name','Automatic Tunning','Selected', 'on','MenuBar', 'none', 'Resize', 'on', 'Position', [500   225   700   549], 'NumberTitle','off');
